@@ -46,12 +46,22 @@ impl<'a> EkkoPacket<'a> {
     pub fn new(buf: &'a mut [u8], pkt: (u16, u16), net: (SocketAddr, SocketAddr)) -> Result<EkkoPacket, EkkoError> {
         match net {
 
-            (SocketAddr::V4(_), SocketAddr::V4(_)) => EkkoPacket::new_ipv4(buf, pkt),
-            (SocketAddr::V6(src), SocketAddr::V6(dst)) => EkkoPacket::new_ipv6(buf, pkt, (src.ip().segments(), dst.ip().segments())),
+            (SocketAddr::V4(_), SocketAddr::V4(_)) => {
+                EkkoPacket::new_ipv4(&mut buf[..], pkt)
+            },
+
+            (SocketAddr::V6(src), SocketAddr::V6(dst)) => {
+                let src_ip = src.ip();
+                let dst_ip = dst.ip();
+
+                EkkoPacket::new_ipv6(&mut buf[..], pkt, {
+                    (src_ip.segments(), dst_ip.segments())
+                })
+            },
+
             (src, dst) => Err(EkkoError::RequestIpMismatch { 
-                src: src.to_string(), 
-                dst: dst.to_string() 
-            })
+                src: src.to_string(), dst: dst.to_string() 
+            }),
         }
     }
 
@@ -62,7 +72,7 @@ impl<'a> EkkoPacket<'a> {
             let mut sum: u32 = data.chunks(2).map(|chunk| match chunk {
                 
                 &[ .. , a, b ] => u16::from_be_bytes([a, b]) as u32,
-                &[ .. , a] => u16::from_be_bytes([a, 0]) as u32,
+                &[ .. , a ] => u16::from_be_bytes([a, 0]) as u32,
                 &[ .. ] => 0 as u32,
                 
             }).sum();
@@ -96,19 +106,23 @@ impl<'a> EkkoPacket<'a> {
             EkkoError::RequestWriteIcmpv4Field("sequence", e.to_string())
         })?;
 
-        for _ in 0..36 {
-            cursor.write_u8(rand::random()).map_err(|e| {
-                EkkoError::RequestWriteIcmpv4Payload(e.to_string())
+        for data in ("Ekko, ekko, ekko ..").chars() {
+            cursor.write_u8(data as u8).map_err(|e| {
+                EkkoError::RequestWriteIcmpv6Payload(e.to_string())
             })?;
         }
 
+        let length = cursor.position();
+
         cursor.set_position(2);
         cursor.write_u16::<BigEndian>(checksum_v4(cursor.get_ref())).map_err(|e| {
             EkkoError::RequestWriteIcmpv4Field("checksum", e.to_string())
         })?;
         
         Ok(EkkoPacket::V4({
-            cursor.into_inner()
+            &(cursor.into_inner()[..({
+                length as usize
+            })])
         }))
     }
     
@@ -127,7 +141,7 @@ impl<'a> EkkoPacket<'a> {
             let mut sum: u32 = data.chunks(2).map(|chunk| match chunk {
                 
                 &[ .. , a, b ] => u16::from_be_bytes([a, b]) as u32,
-                &[ .. , a] => u16::from_be_bytes([a, 0]) as u32,
+                &[ .. , a ] => u16::from_be_bytes([a, 0]) as u32,
                 &[ .. ] => 0 as u32,
                 
             }).sum();
@@ -166,34 +180,49 @@ impl<'a> EkkoPacket<'a> {
             EkkoError::RequestWriteIcmpv6Field("sequence", e.to_string())
         })?;
 
-        for _ in 0..16 {
-            cursor.write_u8(rand::random()).map_err(|e| {
+        for data in ("Ekko, ekko, ekko ..").chars() {
+            cursor.write_u8(data as u8).map_err(|e| {
                 EkkoError::RequestWriteIcmpv6Payload(e.to_string())
             })?;
         }
 
+        let length = cursor.position();
+
         cursor.set_position(2);
         cursor.write_u16::<BigEndian>(checksum_v6(cursor.get_ref(), net)).map_err(|e| {
             EkkoError::RequestWriteIcmpv6Field("checksum", e.to_string())
         })?;
         
         Ok(EkkoPacket::V6({
-            cursor.into_inner()
+            &(cursor.into_inner()[..({
+                length as usize
+            })])
         }))
     }
-
+    
     pub fn as_slice(&self) -> &'a [u8] {
         match self {
 
-            Self::V4(buf) | Self::V6(buf) => buf
+            Self::V4(buf) => &(buf[..]),
+            Self::V6(buf) => &(buf[..]),
         }
     }
 
+    pub fn is_echo_request(&self) -> Result<bool, EkkoError> {
+        Ok(self.get_type()? == 128 || self.get_type()? == 8)
+    }
+
+    #[allow(dead_code)]
+    pub fn is_echo_response(&self) -> Result<bool, EkkoError> {
+        Ok(self.get_type()? == 129 || self.get_type()? == 0)
+    }
+
     pub fn get_type(&self) -> Result<u8, EkkoError> {
         match self {
 
             Self::V4(buf) | Self::V6(buf) => {
                 let mut cursor = Cursor::new(buf);
+                
                 cursor.set_position(0);
                 Ok(cursor.read_u8().map_err(|e| {
                     EkkoError::ResponseReadField("type", e.to_string())
@@ -207,6 +236,7 @@ impl<'a> EkkoPacket<'a> {
 
             Self::V4(buf) | Self::V6(buf) => {
                 let mut cursor = Cursor::new(buf);
+
                 cursor.set_position(1);
                 Ok(cursor.read_u8().map_err(|e| {
                     EkkoError::ResponseReadField("code", e.to_string())
@@ -220,6 +250,7 @@ impl<'a> EkkoPacket<'a> {
 
             Self::V4(buf) | Self::V6(buf) => {
                 let mut cursor = Cursor::new(buf);
+
                 cursor.set_position(2);
                 Ok(cursor.read_u16::<BigEndian>().map_err(|e| {
                     EkkoError::ResponseReadField("checksum", e.to_string())
@@ -237,6 +268,7 @@ impl<'a> EkkoPacket<'a> {
                     8 | 0 => {
 
                         let mut cursor = Cursor::new(buf);
+
                         cursor.set_position(4);
                         Ok(cursor.read_u16::<BigEndian>().map_err(|e| {
                             EkkoError::ResponseReadField("identifier", e.to_string())
@@ -254,6 +286,7 @@ impl<'a> EkkoPacket<'a> {
                     128 | 129 => {
 
                         let mut cursor = Cursor::new(buf);
+
                         cursor.set_position(4);
                         Ok(cursor.read_u16::<BigEndian>().map_err(|e| {
                             EkkoError::ResponseReadField("identifier", e.to_string())
@@ -276,6 +309,7 @@ impl<'a> EkkoPacket<'a> {
                     8 | 0 => {
 
                         let mut cursor = Cursor::new(buf);
+
                         cursor.set_position(6);
                         Ok(cursor.read_u16::<BigEndian>().map_err(|e| {
                             EkkoError::ResponseReadField("sequence number", e.to_string())
@@ -293,6 +327,7 @@ impl<'a> EkkoPacket<'a> {
                     128 | 129 => {
 
                         let mut cursor = Cursor::new(buf);
+
                         cursor.set_position(6);
                         Ok(cursor.read_u16::<BigEndian>().map_err(|e| {
                             EkkoError::ResponseReadField("sequence number", e.to_string())
@@ -315,8 +350,8 @@ impl<'a> EkkoPacket<'a> {
                     3 | 4 | 5 | 11 | 12 => {
 
                         let mut cursor = Cursor::new(buf);
-                        cursor.set_position(8);
 
+                        cursor.set_position(8);
                         let header_octets = ((cursor.read_u8().map_err(|e| {
                             EkkoError::ResponseReadField("internet protocol header size", e.to_string())
                         })? & 0x0F) * 4) as usize;
@@ -403,18 +438,19 @@ impl<'a> EkkoPacket<'a> {
 
                     3 => {
 
-                        let mut cursor = Cursor::new(buf);
-                        cursor.set_position(6);
-                        let value = cursor.read_u16::<BigEndian>().map_err(|e| {
-                            EkkoError::ResponseReadField("problem pointer", e.to_string())
-                        })?;
-
                         Ok(Unreachable::V4(match self.get_code()? {
                             0  => UnreachableCodeV4::DestinationNetworkUnreachable,
                             1  => UnreachableCodeV4::DestinationHostUnreachable,
                             2  => UnreachableCodeV4::DestinationProtocolUnreachable,
                             3  => UnreachableCodeV4::DestinationPortUnreachable,
-                            4  => UnreachableCodeV4::FragmentationRequired(value),
+                            4  => UnreachableCodeV4::FragmentationRequired({
+                                let mut cursor = Cursor::new(buf);
+                                
+                                cursor.set_position(6);
+                                cursor.read_u16::<BigEndian>().map_err(|e| {
+                                    EkkoError::ResponseReadField("problem pointer", e.to_string())
+                                })?
+                            }),
                             5  => UnreachableCodeV4::SourceRouteFailed,
                             6  => UnreachableCodeV4::DestinationNetworkUnknown,
                             7  => UnreachableCodeV4::DestinationHostUnknown,
@@ -473,4 +509,50 @@ impl<'a> Debug for EkkoPacket<'a> {
             .field("code", &(self.get_code()))
             .finish()
     }
-}
\ No newline at end of file
+}
+
+#[cfg(test)]
+mod test {
+    use super::{EkkoPacket};
+    use crate::{EkkoError};
+
+    #[test]
+    fn parse_v4() -> Result<(), EkkoError> {
+        let binary = b"\x0B\x00\xF4\xFF\x00\x00\x00\x00\x45\x60\x00\x2F\x61\x91\x00\x00\x01\x01\x3D\xC6\x00\x00\x00\x00\x08\x08\x08\x08\x08\x00\xF3\x52\x0B\xAD\x00\x01\x45\x6B\x6B\x6F\x2C\x20\x65\x6B\x6B\x6F\x2C\x20\x65\x6B\x6B\x6F\x20\x2E\x2E";
+        let packet = EkkoPacket::V4(binary);
+
+        assert_eq!(packet.get_identifier()?, 0xBAD);
+        assert_eq!(packet.get_checksum()?, 0xF4FF);
+        assert_eq!(packet.get_type()?, 11);
+        assert_eq!(packet.get_code()?, 0);
+
+        let originating_packet = packet.get_originator()?;
+
+        assert_eq!(originating_packet.get_identifier()?, 0xBAD);
+        assert_eq!(originating_packet.get_checksum()?, 0xF352);
+        assert_eq!(originating_packet.get_type()?, 8);
+        assert_eq!(originating_packet.get_code()?, 0);
+
+        Ok(())
+    }
+
+    #[test]
+    fn parse_v6() -> Result<(), EkkoError> {
+        let binary = b"\x03\x00\xB9\x6D\x00\x00\x00\x00\x60\x00\x00\x00\x00\x1B\x3A\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x80\x00\xD5\xD8\x0B\xAD\x00\x01\x45\x6B\x6B\x6F\x2C\x20\x65\x6B\x6B\x6F\x2C\x20\x65\x6B\x6B\x6F\x20\x2E\x2E";
+        let packet = EkkoPacket::V6(binary);
+
+        assert_eq!(packet.get_identifier()?, 0xBAD);
+        assert_eq!(packet.get_checksum()?, 0xB96D);
+        assert_eq!(packet.get_type()?, 3);
+        assert_eq!(packet.get_code()?, 0);
+
+        let originating_packet = packet.get_originator()?;
+
+        assert_eq!(originating_packet.get_identifier()?, 0xBAD);
+        assert_eq!(originating_packet.get_checksum()?, 0xD5D8);
+        assert_eq!(originating_packet.get_type()?, 128);
+        assert_eq!(originating_packet.get_code()?, 0);
+
+        Ok(())
+    }
+}
